@@ -1,5 +1,5 @@
 # engine/__init__.py
-# Copyright (C) 2005-2014 the SQLAlchemy authors and contributors <see AUTHORS file>
+# Copyright (C) 2005-2013 the SQLAlchemy authors and contributors <see AUTHORS file>
 #
 # This module is part of SQLAlchemy and is released under
 # the MIT License: http://www.opensource.org/licenses/mit-license.php
@@ -9,7 +9,7 @@
 The engine package defines the basic components used to interface
 DB-API modules with higher-level statement construction,
 connection-management, execution and result contexts.  The primary
-"entry point" class into this package is the Engine and its public
+"entry point" class into this package is the Engine and it's public
 constructor ``create_engine()``.
 
 This package includes:
@@ -50,14 +50,13 @@ url.py
     within a URL.
 """
 
-# not sure what this was used for
-#import sqlalchemy.databases
-
 from .interfaces import (
-    Compiled,
     Connectable,
     Dialect,
     ExecutionContext,
+
+    # backwards compat
+    Compiled,
     TypeCompiler
 )
 
@@ -83,8 +82,12 @@ from .util import (
     connection_memoize
     )
 
+
 from . import util, strategies
 
+# backwards compat
+from ..sql import ddl
+
 default_strategy = 'plain'
 
 
@@ -93,32 +96,25 @@ def create_engine(*args, **kwargs):
 
     The standard calling form is to send the URL as the
     first positional argument, usually a string
-    that indicates database dialect and connection arguments::
-
-
-        engine = create_engine("postgresql://scott:tiger@localhost/test")
-
+    that indicates database dialect and connection arguments.
     Additional keyword arguments may then follow it which
     establish various options on the resulting :class:`.Engine`
     and its underlying :class:`.Dialect` and :class:`.Pool`
-    constructs::
-
-        engine = create_engine("mysql://scott:tiger@hostname/dbname",
-                                    encoding='latin1', echo=True)
+    constructs.
 
     The string form of the URL is
-    ``dialect[+driver]://user:password@host/dbname[?key=value..]``, where
+    ``dialect+driver://user:password@host/dbname[?key=value..]``, where
     ``dialect`` is a database name such as ``mysql``, ``oracle``,
     ``postgresql``, etc., and ``driver`` the name of a DBAPI, such as
     ``psycopg2``, ``pyodbc``, ``cx_oracle``, etc.  Alternatively,
     the URL can be an instance of :class:`~sqlalchemy.engine.url.URL`.
 
     ``**kwargs`` takes a wide variety of options which are routed
-    towards their appropriate components.  Arguments may be specific to
-    the :class:`.Engine`, the underlying :class:`.Dialect`, as well as the
-    :class:`.Pool`.  Specific dialects also accept keyword arguments that
-    are unique to that dialect.   Here, we describe the parameters
-    that are common to most :func:`.create_engine()` usage.
+    towards their appropriate components.  Arguments may be     specific
+    to the :class:`.Engine`, the underlying :class:`.Dialect`, as well as
+    the     :class:`.Pool`.  Specific dialects also accept keyword
+    arguments that     are unique to that dialect.   Here, we describe the
+    parameters     that are common to most :func:`.create_engine()` usage.
 
     Once established, the newly resulting :class:`.Engine` will
     request a connection from the underlying :class:`.Pool` once
@@ -128,13 +124,11 @@ def create_engine(*args, **kwargs):
     is received.   The :func:`.create_engine` call itself does **not**
     establish any actual DBAPI connections directly.
 
-    .. seealso::
+    See also:
 
-        :doc:`/core/engines`
+    :doc:`/core/engines`
 
-        :doc:`/dialects/index`
-
-        :ref:`connections_toplevel`
+    :ref:`connections_toplevel`
 
     :param case_sensitive=True: if False, result column names
        will match in a case-insensitive fashion, that is,
@@ -354,10 +348,13 @@ def engine_from_config(configuration, prefix='sqlalchemy.', **kwargs):
     arguments.
     """
 
-    opts = util._coerce_config(configuration, prefix)
-    opts.update(kwargs)
-    url = opts.pop('url')
-    return create_engine(url, **opts)
+    options = dict((key[len(prefix):], configuration[key])
+                   for key in configuration
+                   if key.startswith(prefix))
+    options['_coerce_config'] = True
+    options.update(kwargs)
+    url = options.pop('url')
+    return create_engine(url, **options)
 
 
 __all__ = (
